@@ -1,14 +1,14 @@
 from __future__ import annotations
 
 import re
-from datetime import datetime, timedelta, timezone
+from datetime import UTC, datetime, timedelta
 from math import fmod
 from typing import Any
 
 from .singleton_local_time_zone import local_time_zone
 from .time_span import TimeSpan, total_microseconds
 from .time_span import create as create_time_span
-from .types import FSharpRef
+from .types import FSharpRef, float64
 from .util import DateKind
 
 
@@ -56,9 +56,9 @@ def subtract(x: datetime, y: datetime | TimeSpan) -> datetime | TimeSpan:
         # ts.microseconds only contains the microseconds provided to the constructor
         # so we need to calculate the total microseconds ourselves
         delta_microseconds = delta.days * (24 * 3600 * 10**6) + delta.seconds * 10**6 + delta.microseconds
-        return create_time_span(0, 0, 0, 0, 0, delta_microseconds)
+        return create_time_span(float64(0), float64(0), float64(0), float64(0), float64(0), float64(delta_microseconds))
 
-    return x - timedelta(microseconds=total_microseconds(y))
+    return x - timedelta(microseconds=float(total_microseconds(y)))
 
 
 def op_subtraction(x: datetime, y: datetime | TimeSpan) -> datetime | TimeSpan:
@@ -85,7 +85,7 @@ def create(
             minute=m,
             second=s,
             microsecond=mc + ms * 1_000,
-            tzinfo=timezone.utc,
+            tzinfo=UTC,
         )
     else:
         date = datetime(year, month, day, h, m, s, mc + ms * 1_000)
@@ -128,7 +128,7 @@ def microsecond(d: datetime) -> int:
 
 
 def to_universal_time(d: datetime) -> datetime:
-    return d.astimezone(timezone.utc)
+    return d.astimezone(UTC)
 
 
 def day_of_week(d: datetime) -> int:
@@ -211,7 +211,7 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
     cursor_pos = 0
     token_length = 0
     result = ""
-    localized_date = date.astimezone(timezone.utc) if utc else date
+    localized_date = date.astimezone(UTC) if utc else date
 
     while cursor_pos < len(format):
         token = format[cursor_pos]
@@ -227,10 +227,8 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
                         result += localized_date.strftime("%d")
                     case 3:
                         result += short_days[day_of_week(localized_date)]
-                    case 4:
+                    case 4 | _:
                         result += long_days[day_of_week(localized_date)]
-                    case _:
-                        pass
             case "f":
                 token_length = parse_repeat_token(format, cursor_pos, "f")
                 cursor_pos += token_length
@@ -243,7 +241,7 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
                     case 7:
                         result += str(localized_date.microsecond).zfill(6).ljust(token_length, "0")
                     case _:
-                        pass
+                        raise Exception("Input string was not in a correct format.")
             case "F":
                 token_length = parse_repeat_token(format, cursor_pos, "F")
                 cursor_pos += token_length
@@ -261,36 +259,30 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
                         if value != 0:
                             result += str(value).zfill(6).rstrip("0")
                     case _:
-                        pass
+                        raise Exception("Input string was not in a correct format.")
             case "g":
                 token_length = parse_repeat_token(format, cursor_pos, "g")
                 cursor_pos += token_length
-                match token_length:
-                    case 1 | 2:
-                        result += "A.D."
-                    case _:
-                        pass
-
+                result += "A.D."
             case "h":
                 token_length = parse_repeat_token(format, cursor_pos, "h")
                 cursor_pos += token_length
                 match token_length:
                     case 1:
-                        result += str(localized_date.hour % 12)
-                    case 2:
+                        h1Value = localized_date.hour % 12
+                        if h1Value == 0:
+                            h1Value = 12
+                        result += str(h1Value)
+                    case 2 | _:
                         result += localized_date.strftime("%I")
-                    case _:
-                        pass
             case "H":
                 token_length = parse_repeat_token(format, cursor_pos, "H")
                 cursor_pos += token_length
                 match token_length:
                     case 1:
                         result += str(localized_date.hour)
-                    case 2:
+                    case 2 | _:
                         result += localized_date.strftime("%H")
-                    case _:
-                        pass
             case "K":
                 token_length = parse_repeat_token(format, cursor_pos, "K")
                 cursor_pos += token_length
@@ -315,10 +307,8 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
                 match token_length:
                     case 1:
                         result += str(localized_date.minute)
-                    case 2:
+                    case 2 | _:
                         result += localized_date.strftime("%M")
-                    case _:
-                        pass
             case "M":
                 token_length = parse_repeat_token(format, cursor_pos, "M")
                 cursor_pos += token_length
@@ -329,30 +319,24 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
                         result += localized_date.strftime("%m")
                     case 3:
                         result += short_months[month(localized_date) - 1]
-                    case 4:
+                    case 4 | _:
                         result += long_months[month(localized_date) - 1]
-                    case _:
-                        pass
             case "s":
                 token_length = parse_repeat_token(format, cursor_pos, "s")
                 cursor_pos += token_length
                 match token_length:
                     case 1:
                         result += str(localized_date.second)
-                    case 2:
+                    case 2 | _:
                         result += localized_date.strftime("%S")
-                    case _:
-                        pass
             case "t":
                 token_length = parse_repeat_token(format, cursor_pos, "t")
                 cursor_pos += token_length
                 match token_length:
                     case 1:
                         result += localized_date.strftime("%p")[:1]
-                    case 2:
+                    case 2 | _:
                         result += localized_date.strftime("%p")
-                    case _:
-                        pass
             case "y":
                 token_length = parse_repeat_token(format, cursor_pos, "y")
                 cursor_pos += token_length
@@ -419,11 +403,15 @@ def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) ->
 
 
 def date_to_string_with_offset(date: datetime, format: str | None = None) -> str:
-    utc = date.tzinfo == timezone.utc
+    utc = date.tzinfo == UTC
 
     match format:
         case None:
-            raise NotImplementedError("date_to_string_with_offset")
+            # Default format for DateTimeOffset without explicit format
+            if utc:
+                return date.strftime("%Y-%m-%dT%H:%M:%S.%f+00:00")
+            else:
+                return date.strftime("%Y-%m-%dT%H:%M:%S.%f%z")
         case "O" | "o":
             return date.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
         case _ if len(format) == 1:
@@ -433,7 +421,7 @@ def date_to_string_with_offset(date: datetime, format: str | None = None) -> str
 
 
 def date_to_string_with_kind(date: datetime, format: str | None = None) -> str:
-    utc = date.tzinfo == timezone.utc
+    utc = date.tzinfo == UTC
 
     if not format:
         return date.isoformat() if utc else str(date)
@@ -467,7 +455,7 @@ def now() -> datetime:
 
 
 def utc_now() -> datetime:
-    return datetime.utcnow()
+    return datetime.now(UTC).replace(tzinfo=None)
 
 
 def today() -> datetime:
@@ -501,7 +489,7 @@ def min_value() -> datetime:
 
 
 def op_addition(x: datetime, y: TimeSpan) -> datetime:
-    return x + timedelta(microseconds=total_microseconds(y))
+    return x + timedelta(microseconds=float(total_microseconds(y)))
 
 
 def add(x: datetime, y: TimeSpan) -> datetime:
@@ -571,7 +559,7 @@ def parse(string: str) -> datetime:
         adapted_string = re.sub(iso_format_regex, adapt_microseconds, string)
         return datetime.strptime(adapted_string, "%Y-%m-%dT%H:%M:%S.%f")
 
-    raise ValueError("Unsupported format by Fable: %s" % (string))
+    raise ValueError(f"Unsupported format by Fable: {string}")
 
 
 def try_parse(string: str, def_value: FSharpRef[datetime]) -> bool:
@@ -627,31 +615,31 @@ def add_months(d: datetime, v: int) -> datetime:
 
 
 def add_days(d: datetime, v: int) -> datetime:
-    return d + timedelta(days=v)
+    return d + timedelta(days=int(v))
 
 
 def add_hours(d: datetime, v: int) -> datetime:
-    return d + timedelta(hours=v)
+    return d + timedelta(hours=int(v))
 
 
 def add_minutes(d: datetime, v: int) -> datetime:
-    return d + timedelta(minutes=v)
+    return d + timedelta(minutes=int(v))
 
 
 def add_seconds(d: datetime, v: int) -> datetime:
-    return d + timedelta(seconds=v)
+    return d + timedelta(seconds=int(v))
 
 
 def add_milliseconds(d: datetime, v: int) -> datetime:
-    return d + timedelta(milliseconds=v)
+    return d + timedelta(milliseconds=int(v))
 
 
 def add_microseconds(d: datetime, v: int) -> datetime:
-    return d + timedelta(microseconds=v)
+    return d + timedelta(microseconds=int(v))
 
 
 def kind(d: datetime) -> DateKind:
-    if d.tzinfo == timezone.utc:
+    if d.tzinfo == UTC:
         return DateKind.UTC
 
     elif d.tzinfo is None:
@@ -686,7 +674,7 @@ def ticks_to_unix_epoch_microseconds(ticks: int) -> int:
 
 
 def date_offset(d: datetime) -> int:
-    if d.tzinfo == timezone.utc:
+    if d.tzinfo == UTC:
         return 0
     else:
         utc_offset = d.utcoffset()
@@ -705,7 +693,7 @@ def date_offset(d: datetime) -> int:
 
 def create_from_epoch_microseconds(us: int, kind: DateKind | None = None) -> datetime:
     if kind == DateKind.UTC:
-        date = datetime.fromtimestamp(us / 1_000_000, timezone.utc)
+        date = datetime.fromtimestamp(us / 1_000_000, UTC)
     else:
         date = datetime.fromtimestamp(us / 1_000_000)
         if kind == DateKind.Local:
@@ -728,52 +716,60 @@ def from_ticks(ticks: int, kind: DateKind | None = None) -> datetime:
     return date
 
 
+def from_date_time_offset(date_time_offset: Any, _kind: int = 0) -> datetime:
+    """Convert a DateTimeOffset to a DateTime"""
+    # DateTimeOffset is a datetime subclass, so we can just return it as a datetime
+    # The _kind parameter is ignored for now (it's used in .NET for DateTimeKind)
+    return date_time_offset
+
+
 __all__ = [
-    "subtract",
-    "op_subtraction",
-    "create",
-    "year",
-    "month",
-    "day",
-    "hour",
-    "minute",
-    "second",
-    "millisecond",
-    "microsecond",
-    "to_universal_time",
-    "day_of_week",
-    "day_of_year",
-    "to_short_date_string",
-    "to_long_date_string",
-    "to_short_time_string",
-    "to_long_time_string",
-    "to_string",
-    "now",
-    "utc_now",
-    "today",
-    "to_local_time",
-    "compare",
-    "equals",
-    "max_value",
-    "min_value",
-    "op_addition",
     "add",
-    "parse",
-    "try_parse",
-    "date",
-    "is_leap_year",
-    "days_in_month",
-    "add_years",
-    "add_months",
     "add_days",
     "add_hours",
-    "add_minutes",
-    "add_seconds",
-    "add_milliseconds",
     "add_microseconds",
-    "kind",
-    "specify_kind",
-    "ticks",
+    "add_milliseconds",
+    "add_minutes",
+    "add_months",
+    "add_seconds",
+    "add_years",
+    "compare",
+    "create",
+    "date",
     "date_offset",
+    "day",
+    "day_of_week",
+    "day_of_year",
+    "days_in_month",
+    "equals",
+    "from_date_time_offset",
     "from_ticks",
+    "hour",
+    "is_leap_year",
+    "kind",
+    "max_value",
+    "microsecond",
+    "millisecond",
+    "min_value",
+    "minute",
+    "month",
+    "now",
+    "op_addition",
+    "op_subtraction",
+    "parse",
+    "second",
+    "specify_kind",
+    "subtract",
+    "ticks",
+    "to_local_time",
+    "to_long_date_string",
+    "to_long_time_string",
+    "to_short_date_string",
+    "to_short_time_string",
+    "to_string",
+    "to_universal_time",
+    "today",
+    "try_parse",
+    "utc_now",
+    "year",
 ]
