@@ -8,9 +8,9 @@
  * Basically; invariant: date.getTime() always return UTC time.
  */
 
-import { int64, toInt64, toFloat64 } from "./BigInt.js";
-import { FSharpRef } from "./Types.js";
-import { compareDates, DateKind, dateOffset, IDateTime, IDateTimeOffset, padWithZeros } from "./Util.js";
+import { int64, toInt64, toFloat64 } from "./BigInt.ts";
+import { FSharpRef } from "./Types.ts";
+import { Exception, compareDates, DateTimeKind, dateOffset, IDateTime, IDateTimeOffset, padWithZeros } from "./Util.ts";
 
 export type OffsetInMinutes = number;
 export type Offset = "Z" | OffsetInMinutes | null;
@@ -55,7 +55,7 @@ function parseNextChar(format: string, pos: number) {
   return format.charCodeAt(pos + 1);
 }
 
-function parseQuotedString(format: string, pos: number) : [string, number] {
+function parseQuotedString(format: string, pos: number): [string, number] {
   let beginPos = pos;
   // Get the character used to quote the string
   const quoteChar = format[pos];
@@ -75,7 +75,7 @@ function parseQuotedString(format: string, pos: number) : [string, number] {
         result += format[pos];
       } else {
         // This means that '\' is the last character in the string.
-        throw new Error("Invalid string format");
+        throw new Exception("Invalid string format");
       }
     } else {
       result += currentChar;
@@ -84,7 +84,7 @@ function parseQuotedString(format: string, pos: number) : [string, number] {
 
   if (!foundQuote) {
     // We could not find the matching quote
-    throw new Error(`Invalid string format could not find matching quote for ${quoteChar}`);
+    throw new Exception(`Invalid string format could not find matching quote for ${quoteChar}`);
   }
 
   return [result, pos - beginPos + 1];
@@ -94,7 +94,7 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
   let cursorPos = 0;
   let tokenLength = 0;
   let result = "";
-  const localizedDate = utc ? DateTime(date.getTime(), DateKind.UTC) : date;
+  const localizedDate = utc ? DateTime(date.getTime(), DateTimeKind.Utc) : date;
 
   while (cursorPos < format.length) {
     const token = format[cursorPos];
@@ -114,9 +114,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += shortDays[dayOfWeek(localizedDate)];
             break;
           case 4:
-            result += longDays[dayOfWeek(localizedDate)];
-            break;
           default:
+            result += longDays[dayOfWeek(localizedDate)];
             break;
         }
         break;
@@ -132,7 +131,9 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
           // milliseconds provided to it.
           // This is to have the same behavior as .NET when doing:
           // DateTime(1, 2, 3, 4, 5, 6, DateTimeKind.Utc).ToString("fffff") => 00000
-          result += (""+millisecond(localizedDate)).padEnd(tokenLength, "0");
+          result += ("" + millisecond(localizedDate)).padEnd(tokenLength, "0");
+        } else {
+          throw "Input string was not in a correct format.";
         }
         break;
       case "F":
@@ -152,26 +153,27 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
           if (value != 0) {
             result += padWithZeros(value, 3);
           }
+        } else {
+          throw "Input string was not in a correct format.";
         }
         break;
       case "g":
         tokenLength = parseRepeatToken(format, cursorPos, "g");
         cursorPos += tokenLength;
-        if (tokenLength <= 2) {
-          result += "A.D.";
-        }
+        result += "A.D.";
         break;
       case "h":
         tokenLength = parseRepeatToken(format, cursorPos, "h");
         cursorPos += tokenLength;
         switch (tokenLength) {
           case 1:
-            result += hour(localizedDate) % 12
+            const h1Value = hour(localizedDate) % 12;
+            result += h1Value ? h1Value : 12;
             break;
           case 2:
-            result += padWithZeros(hour(localizedDate) % 12, 2);
-            break;
           default:
+            const h2Value = hour(localizedDate) % 12;
+            result += padWithZeros(h2Value ? h2Value : 12, 2);
             break;
         }
         break;
@@ -183,9 +185,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += hour(localizedDate);
             break;
           case 2:
-            result += padWithZeros(hour(localizedDate), 2);
-            break;
           default:
+            result += padWithZeros(hour(localizedDate), 2);
             break;
         }
         break;
@@ -194,14 +195,14 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
         cursorPos += tokenLength;
         switch (tokenLength) {
           case 1:
-            switch (kind(localizedDate)) {
-              case DateKind.UTC:
+            switch (getKind(localizedDate)) {
+              case DateTimeKind.Utc:
                 result += "Z";
                 break;
-              case DateKind.Local:
-                result += dateOffsetToString(localizedDate.getTimezoneOffset() * -60000);
+              case DateTimeKind.Local:
+                result += dateOffsetToString(localizedDate.getTimezoneOffset() * -60_000);
                 break;
-              case DateKind.Unspecified:
+              case DateTimeKind.Unspecified:
                 break;
             }
             break;
@@ -217,9 +218,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += minute(localizedDate);
             break;
           case 2:
-            result += padWithZeros(minute(localizedDate), 2);
-            break;
           default:
+            result += padWithZeros(minute(localizedDate), 2);
             break;
         }
         break;
@@ -237,9 +237,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += shortMonths[month(localizedDate) - 1];
             break;
           case 4:
-            result += longMonths[month(localizedDate) - 1];
-            break;
           default:
+            result += longMonths[month(localizedDate) - 1];
             break;
         }
         break;
@@ -251,9 +250,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += second(localizedDate);
             break;
           case 2:
-            result += padWithZeros(second(localizedDate), 2);
-            break;
           default:
+            result += padWithZeros(second(localizedDate), 2);
             break;
         }
         break;
@@ -265,9 +263,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
             result += localizedDate.getHours() < 12 ? "A" : "P";
             break;
           case 2:
-            result += localizedDate.getHours() < 12 ? "AM" : "PM";
-            break;
           default:
+            result += localizedDate.getHours() < 12 ? "AM" : "PM";
             break;
         }
         break;
@@ -281,16 +278,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
           case 2:
             result += padWithZeros(localizedDate.getFullYear() % 100, 2);
             break;
-          case 3:
-            result += padWithZeros(localizedDate.getFullYear(), 3);
-            break;
-          case 4:
-            result += padWithZeros(localizedDate.getFullYear(), 4);
-            break;
-          case 5:
-            result += padWithZeros(localizedDate.getFullYear(), 5);
-            break;
           default:
+            result += padWithZeros(localizedDate.getFullYear(), tokenLength);
             break;
         }
         break;
@@ -299,15 +288,15 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
         cursorPos += tokenLength;
         let utcOffsetText = "";
 
-        switch (kind(localizedDate)) {
-          case DateKind.UTC:
+        switch (getKind(localizedDate)) {
+          case DateTimeKind.Utc:
             utcOffsetText = "+00:00";
             break;
-          case DateKind.Local:
-            utcOffsetText = dateOffsetToString(localizedDate.getTimezoneOffset() * -60000);
+          case DateTimeKind.Local:
+            utcOffsetText = dateOffsetToString(localizedDate.getTimezoneOffset() * -60_000);
             break;
-          case DateKind.Unspecified:
-            utcOffsetText = dateOffsetToString(toLocalTime(localizedDate).getTimezoneOffset() * -60000);
+          case DateTimeKind.Unspecified:
+            utcOffsetText = dateOffsetToString(toLocalTime(localizedDate).getTimezoneOffset() * -60_000);
             break;
         }
 
@@ -347,7 +336,7 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
           cursorPos += 2;
           result += dateToStringWithCustomFormat(localizedDate, String.fromCharCode(nextChar), utc);
         } else {
-          throw new Error("Invalid format string");
+          throw new Exception("Invalid format string");
         }
         break;
       case "\\":
@@ -356,7 +345,7 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
           cursorPos += 2;
           result += String.fromCharCode(nextChar2);
         } else {
-          throw new Error("Invalid format string");
+          throw new Exception("Invalid format string");
         }
         break;
       default:
@@ -369,8 +358,8 @@ function dateToStringWithCustomFormat(date: Date, format: string, utc: boolean)
   return result;
 }
 
-export function kind(value: IDateTime): number {
-  return value.kind || 0;
+export function getKind(value: IDateTime): DateTimeKind {
+  return value.kind ?? DateTimeKind.Unspecified;
 }
 
 export function unixEpochMillisecondsToTicks(ms: number, offset: number): int64 {
@@ -385,25 +374,18 @@ export function dateOffsetToString(offset: number) {
   const isMinus = offset < 0;
   offset = Math.abs(offset);
   const hours = ~~(offset / 3600000);
-  const minutes = (offset % 3600000) / 60000;
+  const minutes = (offset % 3600000) / 60_000;
   return (isMinus ? "-" : "+") +
     padWithZeros(hours, 2) + ":" +
     padWithZeros(minutes, 2);
 }
 
-export function dateToHalfUTCString(date: IDateTime, half: "first" | "second") {
-  const str = date.toISOString();
-  return half === "first"
-    ? str.substring(0, str.indexOf("T"))
-    : str.substring(str.indexOf("T") + 1, str.length - 1);
-}
-
 function dateToISOString(d: IDateTime, utc: boolean) {
   if (utc) {
     return d.toISOString();
   } else {
     // JS Date is always local
-    const printOffset = d.kind == null ? true : d.kind === DateKind.Local;
+    const printOffset = d.kind == null ? true : d.kind === DateTimeKind.Local;
     return padWithZeros(d.getFullYear(), 4) + "-" +
       padWithZeros(d.getMonth() + 1, 2) + "-" +
       padWithZeros(d.getDate(), 2) + "T" +
@@ -411,7 +393,7 @@ function dateToISOString(d: IDateTime, utc: boolean) {
       padWithZeros(d.getMinutes(), 2) + ":" +
       padWithZeros(d.getSeconds(), 2) + "." +
       padWithZeros(d.getMilliseconds(), 3) +
-      (printOffset ? dateOffsetToString(d.getTimezoneOffset() * -60000) : "");
+      (printOffset ? dateOffsetToString(d.getTimezoneOffset() * -60_000) : "");
   }
 }
 
@@ -426,30 +408,56 @@ function dateToStringWithOffset(date: IDateTimeOffset, format?: string) {
     return d.toISOString().replace(/\.\d+/, "").replace(/[A-Z]|\.\d+/g, " ") + dateOffsetToString((date.offset ?? 0));
   } else if (format.length === 1) {
     switch (format) {
-      case "D": case "d": return dateToHalfUTCString(d, "first");
-      case "T": case "t": return dateToHalfUTCString(d, "second");
+      case "D": return dateToString_D(d);
+      case "d": return dateToString_d(d);
+      case "T": return dateToString_T(toUniversalTime(d));
+      case "t": return dateToString_t(toUniversalTime(d));
       case "O": case "o": return dateToISOStringWithOffset(d, (date.offset ?? 0));
-      default: throw new Error("Unrecognized Date print format");
+      default: throw new Exception("Unrecognized Date print format");
     }
   } else {
     return dateToStringWithCustomFormat(d, format, true);
   }
 }
 
+function dateToString_D(date: IDateTime) {
+  return longDays[dayOfWeek(date)]
+    + ", " + padWithZeros(day(date), 2)
+    + " " + longMonths[month(date) - 1]
+    + " " + year(date);
+}
+
+function dateToString_d(date: IDateTime) {
+  return padWithZeros(month(date), 2)
+    + "/" + padWithZeros(day(date), 2)
+    + "/" + year(date);
+}
+
+function dateToString_T(date: IDateTime) {
+  return padWithZeros(hour(date), 2)
+    + ":" + padWithZeros(minute(date), 2)
+    + ":" + padWithZeros(second(date), 2);
+}
+
+function dateToString_t(date: IDateTime) {
+  return padWithZeros(hour(date), 2)
+    + ":" + padWithZeros(minute(date), 2);
+}
+
 function dateToStringWithKind(date: IDateTime, format?: string) {
-  const utc = date.kind === DateKind.UTC;
+  const utc = date.kind === DateTimeKind.Utc;
   if (typeof format !== "string") {
     return utc ? date.toUTCString() : date.toLocaleString();
   } else if (format.length === 1) {
     switch (format) {
-      case "D": case "d":
-        return utc ? dateToHalfUTCString(date, "first") : date.toLocaleDateString();
-      case "T": case "t":
-        return utc ? dateToHalfUTCString(date, "second") : date.toLocaleTimeString();
+      case "D": return dateToString_D(date);
+      case "d": return dateToString_d(date);
+      case "T": return dateToString_T(date);
+      case "t": return dateToString_t(date);
       case "O": case "o":
         return dateToISOString(date, utc);
       default:
-        throw new Error("Unrecognized Date print format");
+        throw new Exception("Unrecognized Date print format");
     }
   } else {
     return dateToStringWithCustomFormat(date, format, utc);
@@ -462,30 +470,35 @@ export function toString(date: IDateTime | IDateTimeOffset, format?: string, _pr
     : dateToStringWithKind(date, format);
 }
 
-export function DateTime(value: number, kind?: DateKind) {
+export function DateTime(value: number, kind?: DateTimeKind) {
   const d = new Date(value) as IDateTime;
-  d.kind = (kind == null ? DateKind.Unspecified : kind) | 0;
+  d.kind = (kind == null ? DateTimeKind.Unspecified : kind);
   return d;
 }
 
-export function fromTicks(ticks: number | bigint, kind?: DateKind) {
-  kind = kind != null ? kind : DateKind.Local; // better default than Unspecified
+export function fromTicks(ticks: number | bigint, kind?: DateTimeKind) {
+  kind = kind != null ? kind : DateTimeKind.Local; // better default than Unspecified
   let date = DateTime(ticksToUnixEpochMilliseconds(ticks), kind);
 
   // Ticks are local to offset (in this case, either UTC or Local/Unknown).
   // If kind is anything but UTC, that means that the tick number was not
   // in utc, thus getTime() cannot return UTC, and needs to be shifted.
-  if (kind !== DateKind.UTC) {
+  if (kind !== DateTimeKind.Utc) {
     date = DateTime(date.getTime() - dateOffset(date), kind);
   }
 
   return date;
 }
 
-export function fromDateTimeOffset(date: IDateTimeOffset, kind: DateKind) {
+export function fromDateTime(dateOnly: IDateTime, timeOnly: number, kind?: DateTimeKind) {
+  const d = DateTime(dateOnly.getTime() + timeOnly, kind);
+  return DateTime(d.getTime() - dateOffset(d), kind);
+}
+
+export function fromDateTimeOffset(date: IDateTimeOffset, kind: DateTimeKind) {
   switch (kind) {
-    case DateKind.UTC: return DateTime(date.getTime(), DateKind.UTC);
-    case DateKind.Local: return DateTime(date.getTime(), DateKind.Local);
+    case DateTimeKind.Utc: return DateTime(date.getTime(), DateTimeKind.Utc);
+    case DateTimeKind.Local: return DateTime(date.getTime(), DateTimeKind.Local);
     default:
       const d = DateTime(date.getTime() + (date.offset ?? 0), kind);
       return DateTime(d.getTime() - dateOffset(d), kind);
@@ -498,17 +511,17 @@ export function getTicks(date: IDateTime | IDateTimeOffset) {
 
 export function minValue() {
   // This is "0001-01-01T00:00:00.000Z", actual JS min value is -8640000000000000
-  return DateTime(-62135596800000, DateKind.UTC);
+  return DateTime(-62135596800000, DateTimeKind.Utc);
 }
 
 export function maxValue() {
   // This is "9999-12-31T23:59:59.999Z", actual JS max value is 8640000000000000
-  return DateTime(253402300799999, DateKind.UTC);
+  return DateTime(253402300799999, DateTimeKind.Utc);
 }
 
 export function parseRaw(input: string): [Date, Offset] {
   function fail() {
-    throw new Error(`The string is not a valid Date: ${input}`);
+    throw new Exception(`The string is not a valid Date: ${input}`);
   }
 
   if (input == null || input.trim() === "") {
@@ -566,7 +579,7 @@ export function parseRaw(input: string): [Date, Offset] {
       }
       date = new Date(baseDate.getTime() + timeInSeconds * 1000);
       // correct for daylight savings time
-      date = new Date(date.getTime() + (date.getTimezoneOffset() - baseDate.getTimezoneOffset()) * 60000);
+      date = new Date(date.getTime() + (date.getTimezoneOffset() - baseDate.getTimezoneOffset()) * 60_000);
     } else {
       fail();
     }
@@ -585,8 +598,8 @@ export function parse(str: string, detectUTC = false): IDateTime {
   // .NET always parses DateTime as Local if there's offset info (even "Z")
   // Newtonsoft.Json uses UTC if the offset is "Z"
   const kind = offset != null
-    ? (detectUTC && offset === "Z" ? DateKind.UTC : DateKind.Local)
-    : DateKind.Unspecified;
+    ? (detectUTC && offset === "Z" ? DateTimeKind.Utc : DateTimeKind.Local)
+    : DateTimeKind.Unspecified;
   return DateTime(date.getTime(), kind);
 }
 
@@ -602,12 +615,12 @@ export function tryParse(v: string, defValue: FSharpRef<IDateTime>): boolean {
 export function create(
   year: number, month: number, day: number,
   h: number = 0, m: number = 0, s: number = 0,
-  ms: number = 0, kind?: DateKind) {
-  const date = kind === DateKind.UTC
+  ms: number = 0, kind?: DateTimeKind) {
+  const date = kind === DateTimeKind.Utc
     ? new Date(Date.UTC(year, month - 1, day, h, m, s, ms))
     : new Date(year, month - 1, day, h, m, s, ms);
   if (year <= 99) {
-    if (kind === DateKind.UTC) {
+    if (kind === DateTimeKind.Utc) {
       date.setUTCFullYear(year, month - 1, day);
     } else {
       date.setFullYear(year, month - 1, day);
@@ -615,17 +628,17 @@ export function create(
   }
   const dateValue = date.getTime();
   if (isNaN(dateValue)) {
-    throw new Error("The parameters describe an unrepresentable Date.");
+    throw new Exception("The parameters describe an unrepresentable Date.");
   }
   return DateTime(dateValue, kind);
 }
 
 export function now() {
-  return DateTime(Date.now(), DateKind.Local);
+  return DateTime(Date.now(), DateTimeKind.Local);
 }
 
 export function utcNow() {
-  return DateTime(Date.now(), DateKind.UTC);
+  return DateTime(Date.now(), DateTimeKind.Utc);
 }
 
 export function today() {
@@ -643,20 +656,20 @@ export function daysInMonth(year: number, month: number) {
 }
 
 export function toUniversalTime(date: IDateTime) {
-  return date.kind === DateKind.UTC ? date : DateTime(date.getTime(), DateKind.UTC);
+  return date.kind === DateTimeKind.Utc ? date : DateTime(date.getTime(), DateTimeKind.Utc);
 }
 
 export function toLocalTime(date: IDateTime) {
-  return date.kind === DateKind.Local ? date : DateTime(date.getTime(), DateKind.Local);
+  return date.kind === DateTimeKind.Local ? date : DateTime(date.getTime(), DateTimeKind.Local);
 }
 
-export function specifyKind(d: IDateTime, kind: DateKind) {
+export function specifyKind(d: IDateTime, kind: DateTimeKind) {
   return create(year(d), month(d), day(d), hour(d), minute(d), second(d), millisecond(d), kind);
 }
 
 export function timeOfDay(d: IDateTime) {
   return hour(d) * 3600000
-    + minute(d) * 60000
+    + minute(d) * 60_000
     + second(d) * 1000
     + millisecond(d);
 }
@@ -666,35 +679,35 @@ export function date(d: IDateTime) {
 }
 
 export function day(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCDate() : d.getDate();
+  return d.kind === DateTimeKind.Utc ? d.getUTCDate() : d.getDate();
 }
 
 export function hour(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCHours() : d.getHours();
+  return d.kind === DateTimeKind.Utc ? d.getUTCHours() : d.getHours();
 }
 
 export function millisecond(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCMilliseconds() : d.getMilliseconds();
+  return d.kind === DateTimeKind.Utc ? d.getUTCMilliseconds() : d.getMilliseconds();
 }
 
 export function minute(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCMinutes() : d.getMinutes();
+  return d.kind === DateTimeKind.Utc ? d.getUTCMinutes() : d.getMinutes();
 }
 
 export function month(d: IDateTime) {
-  return (d.kind === DateKind.UTC ? d.getUTCMonth() : d.getMonth()) + 1;
+  return (d.kind === DateTimeKind.Utc ? d.getUTCMonth() : d.getMonth()) + 1;
 }
 
 export function second(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCSeconds() : d.getSeconds();
+  return d.kind === DateTimeKind.Utc ? d.getUTCSeconds() : d.getSeconds();
 }
 
 export function year(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCFullYear() : d.getFullYear();
+  return d.kind === DateTimeKind.Utc ? d.getUTCFullYear() : d.getFullYear();
 }
 
 export function dayOfWeek(d: IDateTime) {
-  return d.kind === DateKind.UTC ? d.getUTCDay() : d.getDay();
+  return d.kind === DateTimeKind.Utc ? d.getUTCDay() : d.getDay();
 }
 
 export function dayOfYear(d: IDateTime) {
@@ -709,11 +722,11 @@ export function dayOfYear(d: IDateTime) {
 
 export function add(d: IDateTime, ts: number) {
   const newDate = DateTime(d.getTime() + ts, d.kind);
-  if (d.kind !== DateKind.UTC) {
+  if (d.kind !== DateTimeKind.Utc) {
     const oldTzOffset = d.getTimezoneOffset();
     const newTzOffset = newDate.getTimezoneOffset();
     return oldTzOffset !== newTzOffset
-      ? DateTime(newDate.getTime() + (newTzOffset - oldTzOffset) * 60000, d.kind)
+      ? DateTime(newDate.getTime() + (newTzOffset - oldTzOffset) * 60_000, d.kind)
       : newDate;
   } else {
     return newDate;
@@ -729,7 +742,7 @@ export function addHours(d: IDateTime, v: number) {
 }
 
 export function addMinutes(d: IDateTime, v: number) {
-  return add(d, v * 60000);
+  return add(d, v * 60_000);
 }
 
 export function addSeconds(d: IDateTime, v: number) {
