@@ -1,7 +1,8 @@
-import { toString as dateToString } from "./Date.js";
-import { compare as numericCompare, isNumeric, multiply, Numeric, toExponential, toFixed, toHex, toPrecision } from "./Numeric.js";
-import { escape } from "./RegExp.js";
-import { toString } from "./Types.js";
+import { toString as dateToString } from "./Date.ts";
+import { compare as numericCompare, isNumeric, isIntegral, multiply, Numeric, toExponential, toFixed, toHex, toPrecision } from "./Numeric.ts";
+import { escape } from "./RegExp.ts";
+import { toString } from "./Types.ts";
+import { Exception } from "./Util.ts";
 
 const fsFormatRegExp = /(^|[^%])%([0+\- ]*)(\*|\d+)?(?:\.(\d+))?(\w)/g;
 const interpolateRegExp = /(?:(^|[^%])%([0+\- ]*)(\d+)?(?:\.(\d+))?(\w))?%P\(\)/g;
@@ -11,14 +12,16 @@ function isLessThan(x: Numeric, y: number) {
   return numericCompare(x, y) < 0;
 }
 
-const enum StringComparison {
-  CurrentCulture = 0,
-  CurrentCultureIgnoreCase = 1,
-  InvariantCulture = 2,
-  InvariantCultureIgnoreCase = 3,
-  Ordinal = 4,
-  OrdinalIgnoreCase = 5,
-}
+export const StringComparison = {
+  CurrentCulture: 0,
+  CurrentCultureIgnoreCase: 1,
+  InvariantCulture: 2,
+  InvariantCultureIgnoreCase: 3,
+  Ordinal: 4,
+  OrdinalIgnoreCase: 5,
+} as const;
+
+export type StringComparison = typeof StringComparison[keyof typeof StringComparison];
 
 function cmp(x: string, y: string, ic: boolean | StringComparison) {
   function isIgnoreCase(i: boolean | StringComparison) {
@@ -48,10 +51,10 @@ export function compare(...args: any[]): number {
     case 2: return cmp(args[0], args[1], false);
     case 3: return cmp(args[0], args[1], args[2]);
     case 4: return cmp(args[0], args[1], args[2] === true);
-    case 5: return cmp(args[0].substr(args[1], args[4]), args[2].substr(args[3], args[4]), false);
-    case 6: return cmp(args[0].substr(args[1], args[4]), args[2].substr(args[3], args[4]), args[5]);
-    case 7: return cmp(args[0].substr(args[1], args[4]), args[2].substr(args[3], args[4]), args[5] === true);
-    default: throw new Error("String.compare: Unsupported number of parameters");
+    case 5: return cmp(args[0].slice(args[1], args[1] + args[4]), args[2].slice(args[3], args[3] + args[4]), false);
+    case 6: return cmp(args[0].slice(args[1], args[1] + args[4]), args[2].slice(args[3], args[3] + args[4]), args[5]);
+    case 7: return cmp(args[0].slice(args[1], args[1] + args[4]), args[2].slice(args[3], args[3] + args[4]), args[5] === true);
+    default: throw new Exception("String.compare: Unsupported number of parameters");
   }
 }
 
@@ -68,7 +71,7 @@ export function startsWith(str: string, pattern: string, ic: boolean | StringCom
     return str.startsWith(pattern);
   }
   if (str.length >= pattern.length) {
-    return cmp(str.substr(0, pattern.length), pattern, ic) === 0;
+    return cmp(str.slice(0, pattern.length), pattern, ic) === 0;
   }
   return false;
 }
@@ -78,7 +81,7 @@ export function endsWith(str: string, pattern: string, ic: boolean | StringCompa
     return str.endsWith(pattern);
   }
   if (str.length >= pattern.length) {
-    return cmp(str.substr(str.length - pattern.length, pattern.length), pattern, ic) === 0;
+    return cmp(str.slice(-pattern.length), pattern, ic) === 0;
   }
   return false;
 }
@@ -89,14 +92,14 @@ export function indexOfAny(str: string, anyOf: string[], ...args: number[]) {
   }
   const startIndex = (args.length > 0) ? args[0] : 0;
   if (startIndex < 0) {
-    throw new Error("Start index cannot be negative");
+    throw new Exception("Start index cannot be negative");
   }
   const length = (args.length > 1) ? args[1] : str.length - startIndex;
   if (length < 0) {
-    throw new Error("Length cannot be negative");
+    throw new Exception("Length cannot be negative");
   }
   if (startIndex + length > str.length) {
-    throw new Error("Invalid startIndex and length");
+    throw new Exception("Invalid startIndex and length");
   }
   const endIndex = startIndex + length
   const anyOfAsStr = "".concat.apply("", anyOf);
@@ -166,7 +169,7 @@ export function toText(arg: IPrintfFormat | string) {
 
 export function toFail(arg: IPrintfFormat | string) {
   return continuePrint((x: string) => {
-    throw new Error(x);
+    throw new Exception(x);
   }, arg);
 }
 
@@ -246,7 +249,7 @@ function createPrinter(cont: (...args: any[]) => any, _strParts: string[], _matc
       }
       else if (padLength === "*") {
         if (arg < 0) {
-          throw new Error("Non-negative number required");
+          throw new Exception("Non-negative number required");
         }
         padArg = arg;
         continue;
@@ -295,6 +298,19 @@ export function fsFormat(str: string) {
   };
 }
 
+function splitIntAndDecimalPart(value: string) {
+  let [repInt, repDecimal] = value.split(".");
+  repDecimal === undefined && (repDecimal = "");
+  return {
+    integral: repInt,
+    decimal: repDecimal
+  }
+}
+
+function thousandSeparate(value: string) {
+  return value.replace(/\B(?=(\d{3})+(?!\d))/g, ",");
+}
+
 export function format(str: string | object, ...args: any[]) {
   let str2: string;
   if (typeof str === "object") {
@@ -307,34 +323,92 @@ export function format(str: string | object, ...args: any[]) {
 
   return str2.replace(formatRegExp, (_, idx: number, padLength, format, precision, pattern) => {
     if (idx < 0 || idx >= args.length) {
-      throw new Error("Index must be greater or equal to zero and less than the arguments' length.")
+      throw new Exception("Index must be greater or equal to zero and less than the arguments' length.")
     }
     let rep = args[idx];
+    let parts;
     if (isNumeric(rep)) {
-      precision = precision == null ? null : parseInt(precision, 10);
+      precision = precision == "" ? null : parseInt(precision, 10);
       switch (format) {
-        case "f": case "F":
-          precision = precision != null ? precision : 2;
-          rep = toFixed(rep, precision);
+        case "b": case "B":
+          if (!isIntegral(rep)) {
+            throw new Exception("Format specifier was invalid.");
+          }
+          rep = (rep >>> 0).toString(2).replace(/^0+/, "").padStart(precision || 1, "0");
           break;
-        case "g": case "G":
-          rep = precision != null ? toPrecision(rep, precision) : toPrecision(rep);
+        case "c": case "C":
+          const isNegative = isLessThan(rep, 0);
+          if (isLessThan(rep, 0)) {
+            rep = multiply(rep, -1);
+          }
+          precision = precision == null ? 2 : precision;
+          rep = toFixed(rep, precision);
+          parts = splitIntAndDecimalPart(rep);
+          rep = "¤" + thousandSeparate(parts.integral) + "." + padRight(parts.decimal, precision, "0");
+          if (isNegative) {
+            rep = "(" + rep + ")";
+          }
+          break;
+        case "d": case "D":
+          if (!isIntegral(rep)) {
+            throw new Exception("Format specifier was invalid.");
+          }
+          rep = String(rep);
+          if (precision != null) {
+            if (rep.startsWith("-")) {
+              rep = "-" + padLeft(rep.substring(1), precision, "0");
+            } else {
+              rep = padLeft(rep, precision, "0");
+            }
+          }
           break;
         case "e": case "E":
           rep = precision != null ? toExponential(rep, precision) : toExponential(rep);
           break;
+        case "f": case "F":
+          precision = precision != null ? precision : 2;
+          rep = toFixed(rep, precision);
+          if (precision > 0) {
+            parts = splitIntAndDecimalPart(rep);
+            rep = parts.integral + "." + padRight(parts.decimal, precision, "0");
+          }
+          break;
+        case "g": case "G":
+          rep = precision != null ? toPrecision(rep, precision) : toPrecision(rep);
+          // TODO: Check why some numbers are formatted with decimal part
+          rep = trimEnd(trimEnd(rep, "0"), ".");
+          break;
+        case "n": case "N":
+          precision = precision != null ? precision : 2;
+          rep = toFixed(rep, precision);
+          parts = splitIntAndDecimalPart(rep);
+          rep = thousandSeparate(parts.integral) + "." + padRight(parts.decimal, precision, "0");
+          break;
         case "p": case "P":
           precision = precision != null ? precision : 2;
-          rep = toFixed(multiply(rep, 100), precision) + " %";
-          break;
-        case "d": case "D":
-          rep = precision != null ? padLeft(String(rep), precision, "0") : String(rep);
+          rep = toFixed(multiply(rep, 100), precision)
+          parts = splitIntAndDecimalPart(rep);
+          rep = thousandSeparate(parts.integral) + "." + padRight(parts.decimal, precision, "0") + " %";
           break;
+        case "r": case "R":
+          throw new Exception("The round-trip format is not supported by Fable");
         case "x": case "X":
-          rep = precision != null ? padLeft(toHex(rep), precision, "0") : toHex(rep);
-          if (format === "X") { rep = rep.toUpperCase(); }
+          if (!isIntegral(rep)) {
+            throw new Exception("Format specifier was invalid.");
+          }
+          precision = precision != null ? precision : 2;
+          rep = padLeft(toHex(rep), precision, "0");
+          if (format === "X") {
+            rep = rep.toUpperCase();
+          }
           break;
         default:
+          // If we have format and were not able to handle it throw
+          // See: https://learn.microsoft.com/en-us/dotnet/standard/base-types/standard-numeric-format-strings#standard-format-specifiers
+          if (format) {
+            throw new Exception("Format specifier was invalid.");
+          }
+
           if (pattern) {
             let sign = "";
             rep = (pattern as string).replace(/([0#,]+)(\.[0#]+)?/, (_, intPart: string, decimalPart: string) => {
@@ -390,7 +464,7 @@ export function format(str: string | object, ...args: any[]) {
 
 export function initialize(n: number, f: (i: number) => string) {
   if (n < 0) {
-    throw new Error("String length must be non-negative");
+    throw new Exception("String length must be non-negative");
   }
   const xs = new Array(n);
   for (let i = 0; i < n; i++) {
@@ -401,7 +475,7 @@ export function initialize(n: number, f: (i: number) => string) {
 
 export function insert(str: string, startIndex: number, value: string) {
   if (startIndex < 0 || startIndex > str.length) {
-    throw new Error("startIndex is negative or greater than the length of this instance.");
+    throw new Exception("startIndex is negative or greater than the length of this instance.");
   }
   return str.substring(0, startIndex) + value + str.substring(startIndex);
 }
@@ -429,13 +503,13 @@ export function join<T>(delimiter: string, xs: Iterable<T>): string {
 export function joinWithIndices(delimiter: string, xs: string[], startIndex: number, count: number) {
   const endIndexPlusOne = startIndex + count;
   if (endIndexPlusOne > xs.length) {
-    throw new Error("Index and count must refer to a location within the buffer.");
+    throw new Exception("Index and count must refer to a location within the buffer.");
   }
   return xs.slice(startIndex, endIndexPlusOne).join(delimiter);
 }
 
 function notSupported(name: string): never {
-  throw new Error("The environment doesn't support '" + name + "', please use a polyfill.");
+  throw new Exception("The environment doesn't support '" + name + "', please use a polyfill.");
 }
 
 export function toBase64String(inArray: ArrayLike<number>) {
@@ -474,10 +548,10 @@ export function padRight(str: string, len: number, ch?: string) {
 
 export function remove(str: string, startIndex: number, count?: number) {
   if (startIndex >= str.length) {
-    throw new Error("startIndex must be less than length of string");
+    throw new Exception("startIndex must be less than length of string");
   }
   if (typeof count === "number" && (startIndex + count) > str.length) {
-    throw new Error("Index and count must refer to a location within the string.");
+    throw new Exception("Index and count must refer to a location within the string.");
   }
   return str.slice(0, startIndex) + (typeof count === "number" ? str.substr(startIndex + count) : "");
 }
@@ -492,7 +566,7 @@ export function replicate(n: number, x: string) {
 
 export function getCharAtIndex(input: string, index: number) {
   if (index < 0 || index >= input.length) {
-    throw new Error("Index was outside the bounds of the array.");
+    throw new Exception("Index was outside the bounds of the array.");
   }
   return input[index];
 }
@@ -501,7 +575,7 @@ export function split(str: string, splitters: string[], count?: number, options?
   count = typeof count === "number" ? count : undefined;
   options = typeof options === "number" ? options : 0;
   if (count && count < 0) {
-    throw new Error("Count cannot be less than zero");
+    throw new Exception("Count cannot be less than zero");
   }
   if (count === 0) {
     return [];
@@ -572,7 +646,7 @@ export function filter(pred: (char: string) => boolean, x: string) {
 
 export function substring(str: string, startIndex: number, length?: number) {
   if ((startIndex + (length || 0) > str.length)) {
-    throw new Error("Invalid startIndex and/or length");
+    throw new Exception("Invalid startIndex and/or length");
   }
   return length != null ? str.substr(startIndex, length) : str.substr(startIndex);
 }
