@@ -1,12 +1,40 @@
-// Don't change, this corresponds to DateTime.Kind enum values in .NET
-export const enum DateKind {
-  Unspecified = 0,
-  UTC = 1,
-  Local = 2,
+export type Nullable<T> = T | null | undefined;
+
+export interface MutableArray<T> extends Iterable<T> {
+  readonly length: number;
+  [index: number]: T;
+  every(predicate: (value: T, index: number, array: this) => unknown, thisArg?: any): boolean;
+  fill(value: T, start?: number, end?: number): this;
+  filter(predicate: (value: T, index: number, array: this) => unknown, thisArg?: any): this;
+  find(predicate: (value: T, index: number, array: this) => unknown, thisArg?: any): T | undefined;
+  findIndex(predicate: (value: T, index: number, array: this) => unknown, thisArg?: any): number;
+  forEach(callbackfn: (value: T, index: number, array: this) => void, thisArg?: any): void;
+  indexOf(searchElement: T, fromIndex?: number): number;
+  join(separator?: string): string;
+  lastIndexOf(searchElement: T, fromIndex?: number): number;
+  reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: this) => T): T;
+  reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: this) => T, initialValue: T): T;
+  reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: this) => U, initialValue: U): U;
+  reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: this) => T): T;
+  reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: this) => T, initialValue: T): T;
+  reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: this) => U, initialValue: U): U;
+  reverse(): this;
+  slice(start?: number, end?: number): this;
+  some(predicate: (value: T, index: number, array: this) => unknown, thisArg?: any): boolean;
+  sort(compareFn?: (a: T, b: T) => number): this;
 }
 
+// Don't change, this corresponds to DateTimeKind.Kind enum values in .NET
+export const DateTimeKind = {
+  Unspecified: 0,
+  Utc: 1,
+  Local: 2,
+} as const;
+
+export type DateTimeKind = typeof DateTimeKind[keyof typeof DateTimeKind];
+
 export interface IDateTime extends Date {
-  kind?: DateKind;
+  kind?: DateTimeKind;
 }
 
 export interface IDateTimeOffset extends Date {
@@ -38,20 +66,46 @@ export interface IEqualityComparer<T> {
   GetHashCode(x: T): number;
 }
 
-export interface ICollection<T> extends Iterable<T> {
+export interface ICollection<T> extends IEnumerable<T> {
   readonly Count: number;
   readonly IsReadOnly: boolean;
   Add(item: T): void;
   Clear(): void;
   Contains(item: T): boolean;
-  CopyTo(array: T[], arrayIndex: number): void;
+  CopyTo(array: MutableArray<T>, arrayIndex: number): void;
   Remove(item: T): boolean;
 }
 
+// Exception is intentionally not derived from Error, for performance reasons (see #2160)
+export class Exception {
+  public message: string;
+
+  constructor(msg?: string) {
+    this.message = msg ?? "";
+  }
+}
+
+export function isException(x: any) {
+  return x instanceof Exception || x instanceof Error;
+}
+
+export function isPromise(x: any) {
+  return x instanceof Promise;
+}
+
+export function ensureErrorOrException(e: any): any {
+  // Exceptionally admitting promises as errors for compatibility with React.suspense (see #3298)
+  return (isException(e) || isPromise(e)) ? e : new Exception(String(e));
+}
+
 export function isArrayLike<T>(x: T | ArrayLike<T> | Iterable<T>): x is T[] {
   return Array.isArray(x) || ArrayBuffer.isView(x);
 }
 
+export function isMutableArray<T>(x: T | ArrayLike<T> | Iterable<T>): x is MutableArray<T> {
+  return Array.isArray(x) || ArrayBuffer.isView(x);
+}
+
 export function isIterable<T>(x: T | ArrayLike<T> | Iterable<T>): x is Iterable<T> {
   return x != null && typeof x === "object" && Symbol.iterator in x;
 }
@@ -108,7 +162,8 @@ export interface IEnumerable<T> extends Iterable<T> {
 }
 
 export class Enumerable<T> implements IEnumerable<T> {
-  constructor(private en: IEnumerator<T>) {}
+  private en: IEnumerator<T>;
+  constructor(en: IEnumerator<T>) { this.en = en; }
   public GetEnumerator(): IEnumerator<T> { return this.en; }
   public "System.Collections.IEnumerable.GetEnumerator"(): IEnumerator<any> { return this.en; }
   [Symbol.iterator]() {
@@ -123,7 +178,8 @@ export class Enumerable<T> implements IEnumerable<T> {
 
 export class Enumerator<T> implements IEnumerator<T> {
   private current: T = defaultOf();
-  constructor(private iter: Iterator<T>) { }
+  private iter: Iterator<T>;
+  constructor(iter: Iterator<T>) { this.iter = iter; }
   public ["System.Collections.Generic.IEnumerator`1.get_Current"]() {
     return this.current;
   }
@@ -136,7 +192,7 @@ export class Enumerator<T> implements IEnumerator<T> {
     return !cur.done;
   }
   public ["System.Collections.IEnumerator.Reset"]() {
-    throw new Error("JS iterators cannot be reset");
+    throw new Exception("JS iterators cannot be reset");
   }
   public Dispose() {
     return;
@@ -228,7 +284,7 @@ export function comparerFromEqualityComparer<T>(comparer: IEqualityComparer<T>):
 
 export function assertEqual<T>(actual: T, expected: T, msg?: string): void {
   if (!equals(actual, expected)) {
-    throw Object.assign(new Error(msg || `Expected: ${expected} - Actual: ${actual}`), {
+    throw Object.assign(new Exception(msg || `Expected: ${expected} - Actual: ${actual}`), {
       actual,
       expected,
     });
@@ -237,7 +293,7 @@ export function assertEqual<T>(actual: T, expected: T, msg?: string): void {
 
 export function assertNotEqual<T>(actual: T, expected: T, msg?: string): void {
   if (equals(actual, expected)) {
-    throw Object.assign(new Error(msg || `Expected: ${expected} - Actual: ${actual}`), {
+    throw Object.assign(new Exception(msg || `Expected: ${expected} - Actual: ${actual}`), {
       actual,
       expected,
     });
@@ -284,8 +340,8 @@ export function dateOffset(date: IDateTime | IDateTimeOffset): number {
   const date1 = date as IDateTimeOffset;
   return typeof date1.offset === "number"
     ? date1.offset
-    : ((date as IDateTime).kind === DateKind.UTC
-      ? 0 : date.getTimezoneOffset() * -60000);
+    : ((date as IDateTime).kind === DateTimeKind.Utc
+      ? 0 : date.getTimezoneOffset() * -60_000);
 }
 
 export function int16ToString(i: number, radix?: number) {
@@ -426,7 +482,7 @@ export function safeHash<T>(x: T): number {
   return identityHash(x);
 }
 
-export function equalArraysWith<T>(x: ArrayLike<T>, y: ArrayLike<T>, eq: (x: T, y: T) => boolean): boolean {
+export function equalArraysWith<T>(x: Nullable<ArrayLike<T>>, y: Nullable<ArrayLike<T>>, eq: (x: T, y: T) => boolean): boolean {
   if (x == null) { return y == null; }
   if (y == null) { return false; }
   if (x.length !== y.length) { return false; }
@@ -436,7 +492,7 @@ export function equalArraysWith<T>(x: ArrayLike<T>, y: ArrayLike<T>, eq: (x: T,
   return true;
 }
 
-export function equalArrays<T>(x: ArrayLike<T>, y: ArrayLike<T>): boolean {
+export function equalArrays<T>(x: Nullable<ArrayLike<T>>, y: Nullable<ArrayLike<T>>): boolean {
   return equalArraysWith(x, y, equals);
 }
 
@@ -456,11 +512,11 @@ function equalObjects(x: { [k: string]: any }, y: { [k: string]: any }): boolean
   return true;
 }
 
-export function physicalEquality<T>(x: T, y: T): boolean {
+export function physicalEquals<T>(x: T, y: T): boolean {
   return x === y;
 }
 
-export function equals<T>(x: T, y: T): boolean {
+export function equals<T>(x: Nullable<T>, y: Nullable<T>): boolean {
   if (x === y) {
     return true;
   } else if (x == null) {
@@ -909,7 +965,7 @@ export function curry20<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13,
 }
 
 // More performant method to copy arrays, see #2352
-export function copyToArray<T>(source: T[], sourceIndex: number, target: T[], targetIndex: number, count: number): void {
+export function copyToArray<T>(source: MutableArray<T>, sourceIndex: number, target: MutableArray<T>, targetIndex: number, count: number): void {
   if (ArrayBuffer.isView(source) && ArrayBuffer.isView(target)) {
     (target as any).set((source as any).subarray(sourceIndex, sourceIndex + count), targetIndex);
   } else {
